function [E,iterations] = Mochila( lambda, S, s, eta, ro, c, mu, tita )
%% E = Mochila (lambda,S,s,eta,ro,c,mu,tita)
%% Recibe las tasas de entrada lambda, capacidad de S-Peers S, 
%% tamanos de videos s y parametros de red eta,ro,c,mu y tita.
%% Se deben ingresar los vectores en columnas.


%% Inicializacion
K = length(s);  % cantidad de videos
P = length(S);  % cantidad de super-peers
E = zeros(K,P); % matriz de decisión
W = (tita*s+eta*mu)./(lambda.*s.^2); % vector de deseos
[W,orden] = sort(W,'descend');



%% ------------------------------------------------------------
%% Bloque GreedySeed: 
%% ------------------------------------------------------------
%%   recorre videos en base al vector inicializado con nombre "orden", 
%% y los ubica en los super-peers
for i=1:K
    [basura,memoria] = sort(S,'descend');
    E(orden(i),memoria(1))=1;
    E(orden(i),memoria(2))=1;
    S(memoria(1))= S(memoria(1))-s(orden(i));
    S(memoria(2))= S(memoria(2))-s(orden(i));
end


%% ------------------------------------------------------------

factible = ( restriccion1(s,S,E) & restriccion2(E,P) );
if ( !factible )
    printf( "Solucion inicial no factible! (%i,%i) FIN del COP\n\n", restriccion1(s,S,E), restriccion2(E,P) );
    obj_p2p = 0;
    obj_cdn = 0;
    iterations = 0;
else
    printf( "Solucion inicial generada y factible!\n\n" );
    sirve = 1;
    iterations = 0;



%% ------------------------------------------------------------
%% Bloque LocalSearch    
%% ------------------------------------------------------------
%% En su estado actual visita en orden Agregar, Borrar y Permutar, 
%% hasta que no haya mejoras posibles:
    while( sirve > 0 )
        iterations++;
        
        %Intentar agregar una nueva copia (no mas de 5 intentos)
        sirve1 = 0;
        iter1 = 0;
        while ( sirve1 <= 0 && iter1 < 5 )
            [E,sirve1] = Agregar(lambda,S,s,eta,ro,c,mu,tita,E,K,P);
            iter1++;
        endwhile
        if ( sirve1 > 0 )
            sirve = 1;
            continue;
        endif

        %si no se pudo agregar ninguna copia intentar borrar una copia (no mas de 5 intentos)
        sirve2 = 0;
        iter2 = 0;
        while ( sirve2 <= 0 && iter2 < 5 )
            [E,sirve2] = Borrar(lambda,S,s,eta,ro,c,mu,tita,E,K,P);
            iter2++;
        endwhile
        if ( sirve2 > 0 )
            sirve = 1;
            continue;
        endif

        %si no se pudo agregar ninguna copia ni tampoco se pudo borrar ninguna copia
        % intentar intercambiar 2 copias (no mas de 5 intentos)
        sirve3 = 0;
        iter3 = 0;
        while ( sirve3 <= 0 && iter3 < 5 )
            [E,sirve3] = Permutar(lambda,S,s,eta,ro,c,mu,tita,E,K,P);
            iter3++;
        endwhile
        if ( sirve3 > 0 )
            sirve = 1;
            continue;
        endif

        %si llegamos aqui es porque ninguna mejora pudo ser realizada
        sirve = 0;
    end
%% ------------------------------------------------------------

    printf( "Solucion inicial mejorada %i veces\n\n", iterations );
end %%else solucion inicial factible
end %% de la funcion Mochila


%% ------------------------------------------------------------
%% Funciones para mejorar solucion
%% ------------------------------------------------------------

    function [E,sirve1] = Agregar( lambda, S, s, eta, ro, c, mu, tita, Eviejo, K, P )
%Random = Eviejo-rand(K,P);
z = Eviejo*ones(P,1);
videoscancelados  = (lambda.*s)./(tita*s+c) > (lambda.*s - ro.*z)./(tita.*s+eta*mu); 
if ( length( lambda ) - length( videoscancelados ) > 0 )
    printf( "cantidad de video cancelados: %i \n", length( lambda ) - length( videoscancelados ) );
end
clear z;
Eaux = Eviejo;
Eaux(videoscancelados,:)=1;
Random = Eaux-rand(K,P);
clear Eaux;
[randvideo,randSP] = find( min( min(Random) ) == Random );
E = Eviejo;
E(randvideo,randSP) = 1;

factible = ( restriccion1( s, S, E ) & restriccion2( E,P ) );
mejora   = ( objetivoP2P( lambda, S, s, eta, ro, c, mu, tita, E, P ) < objetivoP2P( lambda, S, s, eta, ro, c, mu, tita, Eviejo, P ) );
sirve1   = mejora & factible;

if ( sirve1 == 0 )
    E = Eviejo;
end
    end


    function [E,sirve2] = Borrar( lambda, S, s, eta, ro, c, mu, tita, Eviejo, K, P )
Random = rand( K, P );
[randv1,randSP1] = find( max( max( Random.* Eviejo ) ) == Random.* Eviejo );
E = Eviejo;
E(randv1,randSP1) = 0;

factible = ( restriccion1( s, S, E ) & restriccion2( E, P ) );
mejora   = ( objetivoP2P( lambda, S, s, eta, ro, c, mu, tita, E, P ) < objetivoP2P( lambda, S, s, eta, ro, c, mu, tita, Eviejo, P ) );
sirve2   = mejora & factible;

if ( sirve2 == 0 )
    E = Eviejo;
end
    end


    function [E,sirve3] = Permutar(lambda, S, s, eta, ro, c, mu, tita, Eviejo, K, P )
Random1 = rand(K,P);
[randv1,randSP1] = find( max( max( Random1 .* Eviejo ) ) == Random1 .* Eviejo );

Random1(randv1,:) = 0;  % aseguro no elegir el mismo video
Random1(:,randSP1) = 0; % aseguro no elegir el mismo par
[randv2,randSP2] = find( max( max( Random1 .* Eviejo ) ) == Random1 .* Eviejo );
E = Eviejo;
E(randv1,randSP1) = 0;
E(randv2,randSP2) = 0;
E(randv1,randSP2) = 1;
E(randv2,randSP1) = 1;

factible = ( restriccion1( s, S, E ) & restriccion2( E,P ) );
mejora   = ( objetivoP2P( lambda, S, s, eta, ro, c, mu, tita, E, P ) < objetivoP2P( lambda, S, s, eta, ro, c, mu, tita, Eviejo, P ) );
sirve3   = mejora & factible;

if ( sirve3 == 0 )
    E = Eviejo;
end
    end

    


