function [Obj_p2p,Obj_cdn,Time_p2p,T_p2p,Time_cdn,T_cdn,Time_ideal,T_ideal,Epj,iterations] = ejecutar_grasp( input_file, P, factor )
% Posicionarse en la carpeta correspondinete 
% (esta sentencia podria no funcionar en Windows)
cd /data/clota/svn/p2ptv/academic/papers/thesis-CR/tests/grasp/
format long

clear s;
clear lambda;
clear Epj;
clear z;

% Cargar el archivo de entrada correspondiente!!
% FORMAT: 3 columnas separadas por ',' (coma)
%    1. numero de secuencia del video
%    2. tamano del video en bytes
%    3. popularidad del video (visitas x segundo)
%
raw = csvread( input_file );
printf( "\n\nDatos de videos cargados desde archivo (%s)\n", input_file );



% datos de videos
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
s = raw(:,2)*8/1000;                   % tamano de los videos (expresados en Kbits)
lambda = factor * raw(:,3);         % popularidad de los videos
K = length( s );                            % cantidad de videos
clear raw;

% datos de usuarios
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
gamma = 100;                             % tasa de abandono de seeders (exponencial de tasa 100)
tita  = 0.01;                                   % tasa de abandono de peers (exponencial de tasa 0.01)
c = 1*1024*1024*8/1000;         % 1 MegaByte de bajada para cada peer (expresado en Kbits)
mu = c/4;                                   % 1/4 MegaByte de subida para cada peer (expresado en Kbits)
eta   = 0.5;                                 % 50% de los peers comparten con otros peers durante la descarga


% datos de superpeers
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rho = 10*1024*1024*8/1000;   % 10 MegaBytes de subida para los super-peers (expresado en Kbits)
%P   = 4;                                      % 4 super-peers
x   = 1500.*1024.*1024*1024.*8/1000; % capacidad de almacenamiento de los super-peers = 1500 GBytes (expresado en Kbits)
S   = ones(P, 1) .* x;                 % todos los super-peers con igual capacidad de almacenamiento


% Verificar viabilidad de los datos de entrada
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
h1 = (tita + gamma) .* s + eta*mu - mu;
[row,col,val] = find( h1 < 0 );
if ( length( row ) > 0 )
    %% eliminar los videos que no verifican las hipotesis
    h1( row ) = [];
    s( row ) = [];
    lambda( row ) = [];
    K = length( s );
end

estabilidad = min( h1 ) > 0;                               % verificar condicion de estabilidad
capacidad = sum(2 * s) <= sum(S);                 % verificar condicion de capacidad minima del sistema

if ( estabilidad & capacidad )
    printf( "\n\nHipotesis verificadas!\n\n" );


%GRASP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
printf( "Informacion inicializada!\n" );
printf( "    Factor de popularidad (factor) = %i\n", factor );
printf( "    Cantidad de videos (K) = %i\n", K );
printf( "    Cantidad de super-peers (P) = %i\n", P );
printf( "    Abandono de peers (gamma) = %f (peers/segundo)\n", gamma );
printf( "    Abandono de seeders (tita) = %f (peers/segundo)\n", tita );
printf( "    Download peers (c) = %f (Kbits)\n", c );
printf( "    Upload peers (mu) = %f (Kbits)\n", mu );
printf( "    Upload super-peers (rho) = %f (Kbits)\n", rho );
printf( "    Storage super-peers (S) = %f (Kbits)\n", x );
printf( "\n\n\n", x );
    
    [Epj,iterations] = Mochila( lambda, S, s, eta, rho, c, mu, tita );

    Obj_p2p = objetivoP2P( lambda, S, s, eta, rho, c, mu, tita, Epj, P );
    Obj_cdn = objetivoCDN( lambda, S, s, eta, rho, c, mu, tita, Epj, P );

    out_file_name = sprintf( "data_youtube/output/result_%s_%d.csv", input_file, factor );
    csvwrite( out_file_name, Epj );
    printf( "GRASP completado! Objetivo: %f.\n    Resultado guargado en %s\n\n\n", Obj_p2p, out_file_name );
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Download Time
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    z = Epj * ones(P,1);
    x_p2p = max( (lambda.*s) ./ (tita*s + c), (lambda.*s - rho.*z) ./ (tita.*s + eta*mu) );
    x_cdn = max( (lambda.*s) ./ (tita*s + c), (lambda.*s - rho.*z) ./ (tita.*s) );

    Time_p2p = sum( x_p2p ) / sum( lambda )
    T_p2p = sum( x_p2p ./ lambda )
    Time_cdn = sum( x_cdn ) / sum( lambda )
    T_cdn = sum( x_cdn ./ lambda )
    
    x_ideal = (lambda ./ s) / c;
    Time_ideal = sum( x_ideal ) / sum( lambda )
    T_ideal = sum( x_ideal ./ lambda )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
else
    printf( "Los datos de entrada no cumplen con las hipotesis del problema!! (%d,%d)\n\n\n",  estabilidad, capacidad );
end
end

