function [Obj_p2p_1,Obj_cdn_1,T_p2p1,T_cdn1,Obj_p2p_2,Obj_cdn_2,T_p2p2,T_cdn2,Obj_p2p_3,Obj_cdn_3,T_p2p3,T_cdn3] = ejecutar_grasp_con_path_relinking( input_file, P, factor )

% Posicionarse en la carpeta correspondinete 
% (esta sentencia podria no funcionar en Windows)
cd /data/clota/svn/p2ptv/academic/papers/thesis-CR/tests/grasp/
format long

clear s;
clear lambda;
clear Epj;
clear z;

% Cargar el archivo de entrada correspondiente!!
% FORMAT: 3 columnas separadas por ',' (coma)
%    1. numero de secuencia del video
%    2. tamano del video en bytes
%    3. popularidad del video (visitas x segundo)
%
raw = csvread( input_file );
printf( " - Datos de videos cargados desde archivo (%s)\n", input_file );



% datos de videos
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
s = raw(:,2)*8/1000;                   % tamano de los videos (expresados en Kbits)
lambda = factor * raw(:,3);         % popularidad de los videos
K = length( s );                            % cantidad de videos
clear raw;

% datos de usuarios
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
gamma = 100;                             % tasa de abandono de seeders (exponencial de tasa 100)
tita  = 0.01;                                   % tasa de abandono de peers (exponencial de tasa 0.01)
c = 1*1024*1024*8/1000;         % 1 MegaByte de bajada para cada peer (expresado en Kbits)
mu = c/4;                                   % 1/4 MegaByte de subida para cada peer (expresado en Kbits)
eta   = 0.5;                                 % 50% de los peers comparten con otros peers durante la descarga


% datos de superpeers
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rho = 10*1024*1024*8/1000;   % 10 MegaBytes de subida para los super-peers (expresado en Kbits)
%P   = 4;                                      % 4 super-peers
x   = 1500.*1024.*1024*1024.*8/1000; % capacidad de almacenamiento de los super-peers = 1500 GBytes (expresado en Kbits)
S   = ones(P, 1) .* x;                 % todos los super-peers con igual capacidad de almacenamiento


% Verificar viabilidad de los datos de entrada
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
h1 = (tita + gamma) .* s + eta*mu - mu;
[row,col,val] = find( h1 < 0 );
if ( length( row ) > 0 )
    %% eliminar los videos que no verifican las hipotesis
    h1( row ) = [];
    s( row ) = [];
    lambda( row ) = [];
    K = length( s );
end

estabilidad = min( h1 ) > 0;                               % verificar condicion de estabilidad
capacidad = sum(2 * s) <= sum(S);                 % verificar condicion de capacidad minima del sistema

if ~( estabilidad & capacidad )
    printf( " - Los datos de entrada no cumplen con las hipotesis del problema!! (%d,%d)\n\n\n",  estabilidad, capacidad );
else
    printf( " - Hipotesis verificadas!\n" );


%GRASP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
printf( " - Informacion inicializada:\n" );
printf( "    Factor de popularidad (factor) = %i\n", factor );
printf( "    Cantidad de videos (K) = %i\n", K );
printf( "    Cantidad de super-peers (P) = %i\n", P );
printf( "    Abandono de peers (gamma) = %f (peers/segundo)\n", gamma );
printf( "    Abandono de seeders (tita) = %f (peers/segundo)\n", tita );
printf( "    Download peers (c) = %f (Kbits)\n", c );
printf( "    Upload peers (mu) = %f (Kbits)\n", mu );
printf( "    Upload super-peers (rho) = %f (Kbits)\n", rho );
printf( "    Storage super-peers (S) = %f (Kbits)\n", x );
printf( "\n\n\n", x );


%%%%%%%%%%%%%%%%%%%%%%%%%% Ejecucion 1 del GRASP %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
printf( "\n\n ---------- Primera ejecucion del GRASP ---------------- \n" );

    %%%%% Solucion 1 %%%%%%%%%%%%%%%%%%%%%    
    [Epj1,iterations1] = Mochila( lambda, S, s, eta, rho, c, mu, tita );

    Obj_p2p_1 = objetivoP2P( lambda, S, s, eta, rho, c, mu, tita, Epj1, P )
    Obj_cdn_1 = objetivoCDN( lambda, S, s, eta, rho, c, mu, tita, Epj1, P )

    z1 = Epj1 * ones(P,1);
    x_p2p1 = max( (lambda.*s) ./ (tita*s + c), (lambda.*s - rho.*z1) ./ (tita.*s + eta*mu) );
    x_cdn1 = max( (lambda.*s) ./ (tita*s + c), (lambda.*s - rho.*z1) ./ (tita.*s) );

    T_p2p1 = sum( x_p2p1 ) / sum( lambda )
    T_cdn1 = sum( x_cdn1 ) / sum( lambda )

    printf( "\n Solucion 1 generada: O_P2P = %f, O_CDN = %f,  T_P2P = %f, T_CDN = %f\n\n", Obj_p2p_1, Obj_cdn_1, T_p2p1, T_cdn1 );
    
    clear z1;
    clear x_p2p1;
    clear x_cdn1;




%%%%%%%%%%%%%%%%%%%%%%%%%% Ejecucion 2 del GRASP %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
printf( "\n\n ---------- Segunda ejecucion del GRASP ---------------- \n" );

    %%%%% Solucion 2 %%%%%%%%%%%%%%%%%%%%%    
    [Epj2,iterations2] = Mochila( lambda, S, s, eta, rho, c, mu, tita );

    Obj_p2p_2 = objetivoP2P( lambda, S, s, eta, rho, c, mu, tita, Epj2, P );
    Obj_cdn_2 = objetivoCDN( lambda, S, s, eta, rho, c, mu, tita, Epj2, P );

    z2 = Epj2 * ones(P,1);
    x_p2p2 = max( (lambda.*s) ./ (tita*s + c), (lambda.*s - rho.*z2) ./ (tita.*s + eta*mu) );
    x_cdn2 = max( (lambda.*s) ./ (tita*s + c), (lambda.*s - rho.*z2) ./ (tita.*s) );

    T_p2p2 = sum( x_p2p2 ) / sum( lambda )
    T_cdn2 = sum( x_cdn2 ) / sum( lambda )

    printf( "\n Solucion 2 generada: O_P2P = %f, O_CDN = %f,  T_P2P = %f, T_CDN = %f\n\n", Obj_p2p_2, Obj_cdn_2, T_p2p2, T_cdn2 );
    
    clear z2;
    clear x_p2p2;
    clear x_cdn2;



%%%%%%%%%%%%%%%%%%%%%%%%%% Ejecucion del PATH-RELINKING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
printf( "\n\n ---------- Ejecucion del PATH-RELINKING ---------------- \n" );

    %%%%%% Comparar las soluciones para determinar el camino %%%%%%%%%%%%%%
    if ( Obj_p2p_1 > Obj_p2p_2 )
        S1 = Epj1;
        S2 = Epj2;
    else
        S1 = Epj2;
        S2 = Epj1;
    end

    Saux  = S1;
    Ebest = S2;
    Obj_best = objetivoP2P( lambda, S, s, eta, rho, c, mu, tita, S2, P );

    %%%%% Fase 1 %%%%%%%%%%%
    counts1 = sum( Saux, 2 );
    counts2 = sum( S2, 2 );
    for i = 1 : K
        if ( counts1(i) < counts2(i) )
            %% Agregar replicas para este video
            for j = 1 : counts2(i)-counts1(i)
                Saux = _add( Saux, i );
                Obj_p2p_aux = objetivoP2P( lambda, S, s, eta, rho, c, mu, tita, Saux, P );
                factible = ( restriccion1( s, S, Saux ) & restriccion2( Saux, P ) );
                if ( factible & Obj_p2p_aux < Obj_best )
                    Ebest = Saux;
                    Obj_best = Obj_p2p_aux;
                end
            end
        end
    end
    
    %%%%% Fase 2 %%%%%%%%%%%
    counts1 = sum( Saux, 2 );
    counts2 = sum( S2, 2 );
    for i = 1 : K
        if ( counts1(i) > counts2(i) )
            %% Eliminar replicas para este video
            for j = 1 : counts1(i)-counts2(i)
                Saux = _del( Saux, i );
                Obj_p2p_aux = objetivoP2P( lambda, S, s, eta, rho, c, mu, tita, Saux, P );
                factible = ( restriccion1( s, S, Saux ) & restriccion2( Saux, P ) );
                if ( factible & Obj_p2p_aux < Obj_best )
                    Ebest = Saux;
                    Obj_best = Obj_p2p_aux;
                end
            end
        end
    end

    Obj_p2p_aux = objetivoP2P( lambda, S, s, eta, rho, c, mu, tita, Saux, P );
    Obj_cdn_aux = objetivoCDN( lambda, S, s, eta, rho, c, mu, tita, Saux, P );

    %%%%% Fase 3 %%%%%%%%%%%
    for i = 1 : K
        x = xor( Saux(i,:), S2(i,:) );
        v = find( x > 0 );
        if ( v > 1 )
            %% Intercambiar replicas para este video
            for j = 1 : 2 : size(v,2)
                Saux = _swap( Saux, i, v(1,j), v(1,j+1) );
                Obj_p2p_aux = objetivoP2P( lambda, S, s, eta, rho, c, mu, tita, Saux, P );
                factible = ( restriccion1( s, S, Saux ) & restriccion2( Saux, P ) );
                if ( factible & Obj_p2p_aux < Obj_best )
                    Ebest = Saux;
                    Obj_best = Obj_p2p_aux;
                end
            end
        end
    end

    Obj_p2p_3 = Obj_best;
    Obj_cdn_3 = objetivoCDN( lambda, S, s, eta, rho, c, mu, tita, Ebest, P );

    z3 = Ebest * ones(P,1);

    x_p2p3 = max( (lambda.*s) ./ (tita*s + c), (lambda.*s - rho.*z3) ./ (tita.*s + eta*mu) );
    x_cdn3 = max( (lambda.*s) ./ (tita*s + c), (lambda.*s - rho.*z3) ./ (tita.*s) );

    T_p2p3 = sum( x_p2p3 ) / sum( lambda )
    T_cdn3 = sum( x_cdn3 ) / sum( lambda )

    printf( "\n Solucion 3 generada: O_P2P = %f, O_CDN = %f,  T_P2P = %f, T_CDN = %f\n\n", Obj_p2p_3, Obj_cdn_3, T_p2p3, T_cdn3 );
    
    clear z3;
    clear x_p2p3;
    clear x_cdn3;

    printf( "\n\n -------------------------------------------------------- \n" );

end
end

